#include "CapacitiveSensor.h"
#include "Average.h"

#define HEAT 0
#define PUMP 1
//#define HEAT 13
//#define PUMP 12
#define IRTRANSISTOR 1
CapacitiveSensor   cs = CapacitiveSensor(4, 3);
#define STATUSHEAT 5

#define MAXBREWTIME 25000 // time to fill one cup
#define WAITAFTERFILTERREMOVED 2500 // wait after filter is removed to clean
#define CLEANTIME 1500 // time to run pump for cleaning the filter
#define LUKEWARMTIME 7000 // time to run pump after heating started again

Average average;

enum State { idle, preFlow, preWait, brewing, brewingLukewarm, brewingCold, waitForFilterRemoved, waitForCleaning, cleaning, done, restart, reBrew };
State state = idle;

unsigned long stateStartTime;
unsigned long cumulatedBrewTime;
unsigned long currBrewTime;
int irSensorReflecting;
#define REMOVETHRESH 25
#define TOUCHTHRESH 10
//#define TOUCHTHRESH 50 // test
//void blinkLed(byte led, byte times){ // poor man's display
//  for (byte i=0; i< times; i++){
//    digitalWrite(led,HIGH);
//    delay (250);
//    digitalWrite(led,LOW);
//    delay (175);
//  }
//}

void setup() {
  //Serial.begin(115200);
  pinMode(HEAT, OUTPUT);
  digitalWrite(HEAT, LOW);
  pinMode(PUMP, OUTPUT);
  digitalWrite(PUMP, LOW);
  pinMode(STATUSHEAT, INPUT);
  pinMode(IRTRANSISTOR, INPUT);
}

void transit(State nextState) {
  state = nextState;
  stateStartTime = millis();
}

void printOnOff(int val) {
  //Serial.println(val ? "ON" : "OFF");
}

void heat(int val) {
  digitalWrite(HEAT, val);
  //Serial.print("HEAT ");
  printOnOff(val);
}

void pump(int val) {
  digitalWrite(PUMP, val);
  //Serial.print("PUMP ");
  printOnOff(val);
}

boolean heating() {
  return digitalRead(STATUSHEAT);
}

boolean sensorTouched() {
  return average.average() > TOUCHTHRESH;
}

boolean filterRemoved() {
  return analogRead(IRTRANSISTOR) < (irSensorReflecting - REMOVETHRESH);
}


boolean interrupting() {
  if (stateStartTime + 1000 > millis()) return false;
  boolean s = sensorTouched();
//  Serial.print("I");
//  Serial.println(s);
  return s;
}

void startPreFlow() {
  //Serial.println("pre");
  transit(preFlow);
  heat(LOW);
  pump(HIGH);
  cumulatedBrewTime = 0;
  irSensorReflecting = analogRead(IRTRANSISTOR);
}

void stopBrewing() {
  //Serial.println("stop");
  heat(HIGH);
  pump(LOW);
  transit(waitForFilterRemoved);
}

void pauseBrewing() {
  pump(LOW);
  cumulatedBrewTime = currBrewTime;
  transit(brewingCold);
}

void resumeBrewing() {
  pump(HIGH);
  transit(brewing);
}

void cleanFilter() {
  pump(HIGH);
  transit(cleaning);
}

void endBrewing() {
  //Serial.println("end");
  pump(LOW);
  transit(done);
}

void loop() {
  average.sample(cs.capacitiveSensor(30));

  switch (state) {
  case idle:
    if (sensorTouched()) startPreFlow();
    break;

  case preFlow:
    if (millis() - stateStartTime >= 2000) {
      pump(LOW);
      transit(preWait);
    }
    break;
    
  case preWait:
    if (interrupting()) {
      stopBrewing();
      break;
    }

    if (millis() - stateStartTime >= 5000) {
      pump(HIGH);
      transit(brewing);
      //Serial.println("end wait");
    }
    break;
    
  case brewing:
    if (interrupting()) {
      stopBrewing();
      break;
    }
    currBrewTime = millis() - stateStartTime + cumulatedBrewTime;
    if (heating()) { if (currBrewTime > MAXBREWTIME) stopBrewing(); }
    else {
      transit(brewingLukewarm);
      cumulatedBrewTime = currBrewTime;
    }
    break;
  
  case brewingLukewarm:
    if (interrupting()) {
      stopBrewing();
      break;
    }
    currBrewTime = millis() - stateStartTime + cumulatedBrewTime;
    if (currBrewTime > MAXBREWTIME) stopBrewing();
    else if (millis() - stateStartTime >= LUKEWARMTIME) pauseBrewing();
    break;

  case brewingCold:
    if (interrupting()) {
      stopBrewing();
      break;
    }
    if (heating()) resumeBrewing();
    break;

  case waitForFilterRemoved:
    if (interrupting()) {
      endBrewing();
      break;
    }
    if (filterRemoved()) transit(waitForCleaning);
    break;

  case waitForCleaning:
    if (interrupting()) {
      endBrewing();
      break;
    }
    if (millis() - stateStartTime >= WAITAFTERFILTERREMOVED) cleanFilter();
    break;

  case cleaning:
    if (interrupting()) {
      endBrewing();
      break;
    }
    if (millis() - stateStartTime >= CLEANTIME) endBrewing();
    break;

  case done:
    if (sensorTouched()) {
      transit(restart);
    }
    break;

  case restart:
    if (!sensorTouched()) startPreFlow();
    else if (millis() - stateStartTime > 1000) {
      transit(reBrew);
      pump(HIGH);
    }
    break;

  case reBrew:
    if (!sensorTouched()) stopBrewing();

  default:
    break;
  }
}
